<?php

namespace bwdsplx_SmartPostList_namespace;

use bwdsplx_SmartPostList_namespace\Traits\Smart_Post_List_Markup;

define("bwdsplx_ASFSK_ASSETS_PUBLIC_DIR_FILE", plugin_dir_url(__FILE__) . "assets/public");
define("bwdsplx_ASFSK_ASSETS_ADMIN_DIR_FILE", plugin_dir_url(__FILE__) . "assets/admin");
define('BWDSPLX_ASFSK_ASSETS__FILE__', __FILE__);
define('BWDSPLX_ASFSK_ASSETS_DIR_PATH', plugin_dir_path(BWDSPLX_ASFSK_ASSETS__FILE__));

require_once( BWDSPLX_ASFSK_ASSETS_DIR_PATH .  'traits/smart-post-list.php' );

class Classbwdsplx_SmartPostList {

    use Smart_Post_List_Markup;

    private static $_instance = null;

    public static function instance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function bwdsplx_admin_editor_scripts() {
        add_filter('script_loader_tag', [$this, 'bwdsplx_admin_editor_scripts_as_a_module'], 10, 2);
    }

    public function bwdsplx_admin_editor_scripts_as_a_module($tag, $handle) {
        if ('bwdsplx_SmartPostList_editor' === $handle) {
            $tag = str_replace('<script', '<script type="module"', $tag);
        }

        return $tag;
    }

    private function include_widgets_files() {
        require_once(__DIR__ . '/widgets/bwdsplx-smart-post-widgets.php');
    }

    public function bwdsplx_register_widgets() {
        // It is now safe to include Widgets files
        $this->include_widgets_files();

        // Register WidgetsF
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Widgets\bwdsplx_SmartPostList_widgets());
    }

    // Register Category
    function bwdsplx_SmartPostList_add_elementor_widget_categories($elements_manager) {

        $elements_manager->add_category(
            'bwdsplx-SmartPostList-category',
            [
                'title' => esc_html__('Smart Post List', 'smart-list-post'),
                'icon' => 'eicon-person',
            ]
        );
    }

    //css-js-link-here
    public function bwdsplx_all_assets_for_the_public() {
        wp_enqueue_script('bwdsplx-SmartPostList-nice', plugin_dir_url(__FILE__) . 'assets/public/js/jquery.nice-select.min.js', [ 'jquery' ], Smart_Post_List_VERSION, true);

        wp_enqueue_script('bwdsplx-SmartPostList-pro', plugin_dir_url(__FILE__) . 'assets/public/js/main.js', [ 'jquery' ], Smart_Post_List_VERSION, true);
		
        $all_css_file = array(
            'bwdsplx-SmartPostList-plugin-nice' => array('bwdsplx-SmartPostList_css_path_define' => bwdsplx_ASFSK_ASSETS_PUBLIC_DIR_FILE . '/css/nice-select.css'),
            'bwdsplx-SmartPostList-plugin-main' => array('bwdsplx-SmartPostList_css_path_define' => bwdsplx_ASFSK_ASSETS_PUBLIC_DIR_FILE . '/css/main.css'),
        );
        foreach ($all_css_file as $handle => $fileinfo) {
            wp_enqueue_style($handle, $fileinfo['bwdsplx-SmartPostList_css_path_define'], null, Smart_Post_List_VERSION, 'all');
        }
    }

    //admin-icon
    public function bwdsplx_all_assets_for_elementor_editor_admin() {
        $all_css_js_file = array(
            'bwdsplx-SmartPostList-admin-main' => array('bwdsplx_path_admin_define' => bwdsplx_ASFSK_ASSETS_ADMIN_DIR_FILE . '/css/icon.css'),
        );
        foreach ($all_css_js_file as $handle => $fileinfo) {
            wp_enqueue_style($handle, $fileinfo['bwdsplx_path_admin_define'], null, Smart_Post_List_VERSION, 'all');
        }
    }

    /**
     * Smart Post List Ajax Handler
     */
    public static function smart_post_list_ajax() {

        $security = check_ajax_referer('bwd_smart_post_nonce', 'security');

        if (true == $security && isset($_POST['querySettings'])) :

            $settings = $_POST['querySettings'];

            $list_column = $settings['list_column'];
            $class_array = [];
            if ('yes' === $settings['bwdsplx_make_featured_post']) {
                $class_array['featured'] = 'bwdsplx-spl-column bwdsplx-spl-featured-post-wrap ' . esc_attr($settings['bwdsplx_featured_post_column']);
                $class_array['featured_inner'] = 'bwdsplx-spl-featured-post ' . 'bwdsplx-spl-featured-' . esc_attr($settings['bwdsplx_featured_post_style']);
            }

            $per_page = $settings['per_page'];
            $args = $settings['args'];
            $args['posts_per_page'] = $per_page;

            if ($_POST['offset']) {
                $args['offset'] = $_POST['offset'];
            }
            if ($_POST['termId'] && is_numeric($_POST['termId'])) {
                $args['tax_query'] = array(
                    array(
                        'taxonomy' => '',
                        'field' => 'term_taxonomy_id',
                        'terms' => $_POST['termId'],
                    ),
                );
            }

            $args['suppress_filters'] = false;

            $posts = get_posts($args);
            $loop = 1;

            if (count($posts) !== 0) {

                self::render_spl_markup($settings, $posts, $class_array, $list_column, $per_page);
            }

        endif;
        wp_die();
    }

    public function register_controls( $controls_manager = null ) {
		include_once( BWDSPLX_ASFSK_ASSETS_DIR_PATH . 'controls/lazy-select.php' );

		$lazy_select = __NAMESPACE__ . '\Controls\Lazy_Select';

		$controls_manager->register( new $lazy_select() );
	}

    public function include_files(){
        include_once( BWDSPLX_ASFSK_ASSETS_DIR_PATH . 'classes/lazy-query-manager.php' );
        include_once( BWDSPLX_ASFSK_ASSETS_DIR_PATH . 'classes/assets-manager.php');
        // include_once( BWDSPLX_ASFSK_ASSETS_DIR_PATH . 'traits/smart-post-list.php' );
    }

    public function __construct() {

        add_action( 'elementor/controls/register', [$this, 'register_controls'] );
		// For public assets
		add_action('wp_enqueue_scripts', [$this, 'bwdsplx_all_assets_for_the_public']);
	
		// For Elementor Editor
		add_action('elementor/editor/before_enqueue_scripts', [$this, 'bwdsplx_all_assets_for_elementor_editor_admin']);
	
		// Register Category
		add_action( 'elementor/elements/categories_registered', [ $this, 'bwdsplx_SmartPostList_add_elementor_widget_categories' ] );
	
		// Register widgets
		add_action( 'elementor/widgets/widgets_registered', [ $this, 'bwdsplx_register_widgets' ] );
	
		// Register editor scripts
		add_action( 'elementor/editor/after_enqueue_scripts', [ $this, 'bwdsplx_admin_editor_scripts' ] );
	
		add_action('wp_ajax_bwdsplx_smart_post_list_action', [__CLASS__, 'smart_post_list_ajax']);
		add_action('wp_ajax_nopriv_bwdsplx_smart_post_list_action', [__CLASS__, 'smart_post_list_ajax']);
	
        $this->include_files();

	}
	
}

// Instantiate Plugin Class
Classbwdsplx_SmartPostList::instance();
