<?php
namespace bwdsplx_SmartPostList_namespace\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use bwdsplx_SmartPostList_namespace\Controls\Lazy_Select;
use bwdsplx_SmartPostList_namespace\Lazy_Query_Manager;
use bwdsplx_SmartPostList_namespace\Traits\Smart_Post_List_Markup;


if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class bwdsplx_SmartPostList_widgets extends Widget_Base {

	use Smart_Post_List_Markup;

    public function get_name() {
		return esc_html__( 'bwdsplx-smart-post-list', 'smart-list-post' );
	}

	public function get_title() {
		return esc_html__( 'Smart Post List', 'smart-list-post' );
	}

	public function get_icon() {
		return 'bwdsplx-SmartPostList-icon eicon-posts-group';
	}

	public function get_categories() {
		return [ 'bwdsplx-SmartPostList-category' ];
	}

    public function conditions ($key) {
		$condition = [
			'list_post_show' => [
				'relation' => 'or',
				'terms' => [
					[
						'name' => 'bwdsplx_make_featured_post',
						'operator' => '!==',
						'value' => 'yes',
					],
					[
						'relation' => 'and',
						'terms' => [
							[
								'name' => 'bwdsplx_make_featured_post',
								'operator' => '===',
								'value' => 'yes',
							],
							[
								'name' => 'bwdsplx_featured_post_column',
								'operator' => '!==',
								'value' => 'featured-col-2',
							],
							[
								'name' => 'column',
								'operator' => 'in',
								'value' => ['col-2'],
							],
						],
					],
					[
						'relation' => 'and',
						'terms' => [
							[
								'name' => 'bwdsplx_make_featured_post',
								'operator' => '===',
								'value' => 'yes',
							],
							[
								'name' => 'column',
								'operator' => 'in',
								'value' => ['col-3'],
							],
						],
					],
				],
			],
			'list_post_meta_old_style' => [
				'terms' => [
					[
						'name' => 'bwdsplx_list_meta_active[0]',
						'operator' => 'in',
						'value' =>['author','date','comments'],
					],
				],
			],
			'feature_meta_old_style' => [
				'terms' => [
					[
						'name' => 'bwdsplx_featured_meta_active[0]',
						'operator' => 'in',
						'value' => ['author','date','comments'],
					],
				],
			],
			'list_post_meta_style' => [
				'terms' => [
					[
						'name' => 'bwdsplx_list_meta_active',
						'operator' => '!=',
						'value' => '',
					],
				],
			],
			'feature_meta_style' => [
				'terms' => [
					[
						'name' => 'bwdsplx_featured_meta_active',
						'operator' => '!=',
						'value' => '',
					],
				],
			],
			'feature_item_height' => [
				'relation' => 'or',
				'terms' => [
					[
						'relation' => 'and',
						'terms' => [
							[
								'name' => 'bwdsplx_make_featured_post',
								'operator' => '===',
								'value' => 'yes',
							],
							[
								'name' => 'bwdsplx_featured_post_style',
								'operator' => '===',
								'value' => 'inside-conent',
							],
							[
								'name' => 'column',
								'operator' => 'in',
								'value' => ['col-1'],
							],
						],
					],
					[
						'relation' => 'and',
						'terms' => [
							[
								'name' => 'bwdsplx_make_featured_post',
								'operator' => '===',
								'value' => 'yes',
							],
							[
								'name' => 'bwdsplx_featured_post_column',
								'operator' => '===',
								'value' => 'featured-col-2',
							],
							[
								'name' => 'bwdsplx_featured_post_style',
								'operator' => '===',
								'value' => 'inside-conent',
							],
							[
								'name' => 'column',
								'operator' => 'in',
								'value' => ['col-2'],
							],
						],
					],
				],
			],
		];

		return $condition[$key];
	}

    protected function register_controls(){
		/**
		 * Register widget content controls
		*/
		$this->__layout_content_controls();
		$this->__top_bar_content_controls();
		$this->__featured_post_content_controls();
		$this->__list_post_content_controls();
		$this->__query_content_controls();

		/**
		 * Register widget style controls
		*/
		$this->__layout_style_controls();
		$this->__top_bar_style_controls();
		$this->__top_bar_title_style_controls();
		$this->__top_bar_filter_style_controls();
		$this->__top_bar_navigation_style_controls();
		$this->__bwdsplx_featured_post_style_controls();
		$this->__featured_post_badge_style_controls();
		$this->__list_post_style_controls();
	}

	//Layout Settings
	protected function __layout_content_controls() {

		//Layout Settings
		$this->start_controls_section(
			'bwdsplx_section_spl_layout',
			[
				'label' => __( 'Layout', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'column',
			[
				'label' => __( 'Column', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'col-3',
				'options' => [
					'col-1' => __('Column 1', 'bwdsplx-post-smart'),
					'col-2' => __('Column 2', 'bwdsplx-post-smart'),
					'col-3' => __('Column 3', 'bwdsplx-post-smart'),
				],
			]
		);

		$this->add_control(
			'bwdsplx_top_bar_show',
			[
				'label' => __( 'Show Top Bar', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'bwdsplx_make_featured_post',
			[
				'label' => __( 'First Post Featured', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'bwdsplx_featured_post_column',
			[
				'label' => __( 'Featured Post Column', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'featured-col-1',
				'options' => [
					'featured-col-1' => __('Column 1', 'bwdsplx-post-smart'),
					'featured-col-2' => __('Column 2', 'bwdsplx-post-smart'),
				],
				'condition' => [
					'column!' => 'col-1',
					'bwdsplx_make_featured_post' => 'yes',
				]
			]
		);

		$this->end_controls_section();
	}

	//Top Bar Settings
	protected function __top_bar_content_controls() {

		//Top Bar Settings
		$this->start_controls_section(
			'bwdsplx_section_spl_top_bar',
			[
				'label' => __( 'Top Bar', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
				'condition' => [
					'bwdsplx_top_bar_show' => 'yes',
				],
			]
		);

		$this->add_control(
			'bwdsplx_widget_title',
			[
				'label' => __( 'Title', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => 'Trending Articles',
			]
		);

		$this->add_control(
			'bwdsplx_widget_title_tag',
			[
				'label' => __( 'Title Tag', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'h4',
				'options' => [
					'h1' => __( 'H1', 'bwdsplx-post-smart' ),
					'h2' => __( 'H2', 'bwdsplx-post-smart' ),
					'h3' => __( 'H3', 'bwdsplx-post-smart' ),
					'h4' => __( 'H4', 'bwdsplx-post-smart' ),
					'h5' => __( 'H5', 'bwdsplx-post-smart' ),
					'h6' => __( 'H6', 'bwdsplx-post-smart' ),
					'div' => __( 'DIV', 'bwdsplx-post-smart' ),
				],
				'condition' => [
					'bwdsplx_widget_title!' => '',
				],
				'separator' => 'after',
			]
		);

		$this->add_control(
			'bwdsplx_navigation_show',
			[
				'label' => __( 'Navigation', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'bwdsplx_category_filter',
			[
				'label' => __( 'Show Filter', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'bwdsplx_category_filter_style',
			[
				'label' => __( 'Filter Style', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'dropdown',
				'options' => [
					'inline' => __('Inline', 'bwdsplx-post-smart'),
					'dropdown' => __('Dropdown', 'bwdsplx-post-smart'),
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_control(
			'bwdsplx_filter_terms_ids',
			[
				'label' => __( 'Filter Terms', 'bwdsplx-post-smart' ),
				'description' => __( 'Terms are items in a taxonomy. The available taxonomies are: Categories, Tags, Formats and custom taxonomies.Select terms for selected filter item.by default, it will show all taxonomy under selected post type in the query settings.', 'bwdsplx-post-smart' ),
				'type' => Lazy_Select::TYPE,
				'multiple' => true,
				'label_block' => true,
				'placeholder' => __( 'Type and select terms', 'bwdsplx-post-smart' ),
				'lazy_args' => [
					'query' => Lazy_Query_Manager::QUERY_TERMS,
					'widget_props' => [
						'post_type' => 'bwdsplx_posts_post_type'
					]
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_control(
			'bwdsplx_filter_all_text',
			[
				'label' => __( 'All Text', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' =>  __( 'All', 'bwdsplx-post-smart' ),
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->end_controls_section();

	}

	//Featured Post Settings
	protected function __featured_post_content_controls() {

		//Featured Post Settings
		$this->start_controls_section(
			'bwdsplx_section_spl_featured_post',
			[
				'label' => __( 'Featured Post', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
				'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
				]
			]
		);

		$this->add_control(
			'bwdsplx_featured_post_style',
			[
				'label' => __( 'Style', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'inside-conent',
				'options' => [
					'inside-conent' => __('Content Inside', 'bwdsplx-post-smart'),
					'outside-conent' => __('Content Outside', 'bwdsplx-post-smart'),
				],
				'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
				]
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Image_Size::get_type(),
			[
				'name' => 'featured_image',
				'default' => 'thumbnail',
				'exclude' => [
					'custom'
				],
				'condition' => [
					'bwdsplx_make_featured_post' => 'yes'
				]
			]
		);

		$this->add_control(
			'bwdsplx_show_badge',
			[
				'label' => __( 'Badge', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => __( 'Show', 'bwdsplx-post-smart' ),
				'label_off' => __( 'Hide', 'bwdsplx-post-smart' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'bwdsplx_taxonomy_badge',
			[
				'label' => __( 'Badge Taxonomy', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'label_block' => true,
				'default' => 'category',
				'options' => bwdsplx_pro_get_taxonomies(),
				'condition' => [
					'bwdsplx_show_badge' => 'yes',
				],
			]
		);

		$this->add_control(
			'bwdsplx_featured_post_title',
			[
				'label' => __( 'Title', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => __( 'Show', 'bwdsplx-post-smart' ),
				'label_off' => __( 'Hide', 'bwdsplx-post-smart' ),
				'return_value' => 'yes',
				'default' => 'yes',
				'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
				]
			]
		);

		$this->add_control(
			'bwdsplx_featured_post_title_tag',
			[
				'label' => __( 'Title Tag', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'h4',
				'options' => [
					'h1' => __( 'H1', 'bwdsplx-post-smart' ),
					'h2' => __( 'H2', 'bwdsplx-post-smart' ),
					'h3' => __( 'H3', 'bwdsplx-post-smart' ),
					'h4' => __( 'H4', 'bwdsplx-post-smart' ),
					'h5' => __( 'H5', 'bwdsplx-post-smart' ),
					'h6' => __( 'H6', 'bwdsplx-post-smart' ),
					'div' => __( 'DIV', 'bwdsplx-post-smart' ),
				],
				'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
					'bwdsplx_featured_post_title' => 'yes'
				],
			]
		);

		$this->add_control(
			'bwdsplx_featured_meta_active',
			[
				'type' => \Elementor\Controls_Manager::SELECT2,
				'label' => __( 'Active Meta', 'bwdsplx-post-smart' ),
				'description' => __( 'Select to show and unselect to hide', 'bwdsplx-post-smart' ),
				'label_block' => true,
				'multiple' => true,
				'default' => ['author', 'date', 'comments'],
				'options' => [
					'author' => __( 'Author', 'bwdsplx-post-smart' ),
					'date' => __( 'Date', 'bwdsplx-post-smart' ),
					'comments' => __( 'Comments', 'bwdsplx-post-smart' ),
				]
			]
		);

		$this->add_control(
            'bwdsplx_featured_post_author_icon',
            [
                'label' => esc_html__( 'Author Icon', 'bwdsplx-post-smart' ),
                'type' => \Elementor\Controls_Manager::ICONS,
                'label_block' => false,
                'skin' => 'inline',
                'default' => [
                    'value' => 'fas fa-user',
                    'library' => 'fa-solid',
                ],
                'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
					'bwdsplx_featured_meta_active' => 'author',
                ],
            ]
        );

		$this->add_control(
            'bwdsplx_featured_post_date_icon',
            [
                'label' => esc_html__( 'Date Icon', 'bwdsplx-post-smart' ),
                'type' => \Elementor\Controls_Manager::ICONS,
                'label_block' => false,
                'skin' => 'inline',
                'default' => [
                    'value' => 'fas fa-calendar-alt',
                    'library' => 'fa-solid',
				],
                'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
					'bwdsplx_featured_meta_active' => 'date',
                ],
            ]
        );

		$this->add_control(
            'bwdsplx_featured_post_comment_icon',
            [
                'label' => esc_html__( 'Comment Icon', 'bwdsplx-post-smart' ),
                'type' => \Elementor\Controls_Manager::ICONS,
                'label_block' => false,
                'skin' => 'inline',
                'default' => [
                    'value' => 'fas fa-comments',
                    'library' => 'fa-solid',
				],
                'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
					'bwdsplx_featured_meta_active' => 'comments',
                ],
            ]
		);

		$this->add_control(
			'bwdsplx_featured_excerpt_length',
			[
				'type' => \Elementor\Controls_Manager::NUMBER,
				'label' => __( 'Excerpt Length', 'bwdsplx-post-smart' ),
				'min' => 0,
				'default' => 15,
				'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
				]
			]
		);

		$this->add_control(
			'bwdsplx_featured_post_align',
			[
				'label' => __( 'Alignment', 'bwdsplx-post-smart' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::CHOOSE,
				'default' => 'bottom',
				'options' => [
					'top' => [
						'title' => __( 'Top', 'bwdsplx-post-smart' ),
						'icon' => 'eicon-v-align-top',
					],
					'middle' => [
						'title' => __( 'Middle', 'bwdsplx-post-smart' ),
						'icon' => 'eicon-v-align-middle',
					],
					'bottom' => [
						'title' => __( 'Bottom', 'bwdsplx-post-smart' ),
						'icon' => 'eicon-v-align-bottom',
					],
				],
				'style_transfer' => true,
				'selectors_dictionary' => [
					'top' => '-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start;',
					'middle' => '-webkit-box-align:center;-ms-flex-align:center;align-items:center;',
					'bottom' => '-webkit-box-align:end;-ms-flex-align:end;align-items:flex-end;',
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-inside-conent' => '{{VALUE}};',
					'{{WRAPPER}} .bwdsplx-spl-featured-outside-conent' => '{{VALUE}};',
				],
			]
		);

		$this->end_controls_section();

	}

	//List Post Settings
	protected function __list_post_content_controls() {

		//List Post Settings
		$this->start_controls_section(
			'bwdsplx_section_spl_list_post',
			[
				'label' => __( 'List Post', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
				'conditions' => $this->conditions('list_post_show'),
			]
		);

		$this->add_control(
			'bwdsplx_list_post_title_tag',
			[
				'label' => __( 'Title Tag', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'h4',
				'options' => [
					'h1' => __( 'H1', 'bwdsplx-post-smart' ),
					'h2' => __( 'H2', 'bwdsplx-post-smart' ),
					'h3' => __( 'H3', 'bwdsplx-post-smart' ),
					'h4' => __( 'H4', 'bwdsplx-post-smart' ),
					'h5' => __( 'H5', 'bwdsplx-post-smart' ),
					'h6' => __( 'H6', 'bwdsplx-post-smart' ),
					'div' => __( 'DIV', 'bwdsplx-post-smart' ),
				],
			]
		);

		$this->add_control(
			'bwdsplx_list_post_image',
			[
				'label' => __( 'Image', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => __( 'Show', 'bwdsplx-post-smart' ),
				'label_off' => __( 'Hide', 'bwdsplx-post-smart' ),
				'return_value' => 'yes',
				'default' => 'yes',
                'conditions' => [
                    'relation' => 'or',
                    'terms' => [
                        [
                            'name' => 'bwdsplx_make_featured_post',
                            'operator' => '!==',
                            'value' => 'yes',
                        ],
                        [
                            'relation' => 'and',
                            'terms' => [
                                [
                                    'name' => 'bwdsplx_make_featured_post',
                                    'operator' => '===',
                                    'value' => 'yes',
                                ],
                                [
                                    'name' => 'column',
                                    'operator' => 'in',
                                    'value' => ['col-1','col-2','col-3'],
                                ],
                            ],
                        ]
                    ],
                ],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Image_Size::get_type(),
			[
				'name' => 'bwdsplx_list_post_image',
				'default' => 'thumbnail',
				'exclude' => [
					'custom'
				],
				'condition' => [
					'bwdsplx_list_post_image' => 'yes'
				]
			]
		);

		$this->add_control(
			'bwdsplx_list_meta_active',
			[
				'type' => \Elementor\Controls_Manager::SELECT2,
				'label' => __( 'Active Meta', 'bwdsplx-post-smart' ),
				'description' => __( 'Select to show and unselect to hide', 'bwdsplx-post-smart' ),
				'label_block' => true,
				'multiple' => true,
				'default' => ['author'],
				'options' => [
					'author' => __( 'Author', 'bwdsplx-post-smart' ),
					'date' => __( 'Date', 'bwdsplx-post-smart' ),
					'comments' => __( 'Comments', 'bwdsplx-post-smart' ),
				]
			]
		);

		$this->add_control(
            'bwdsplx_list_post_author_icon',
            [
                'label' => esc_html__( 'Author Icon', 'bwdsplx-post-smart' ),
                'type' => \Elementor\Controls_Manager::ICONS,
                'label_block' => false,
                'skin' => 'inline',
                'default' => [
                    'value' => 'fas fa-user',
                    'library' => 'fa-solid',
				],
                'condition' => [
					'bwdsplx_list_meta_active' => 'author',
                ],
            ]
        );

		$this->add_control(
            'bwdsplx_list_post_date_icon',
            [
                'label' => esc_html__( 'Date Icon', 'bwdsplx-post-smart' ),
                'type' => \Elementor\Controls_Manager::ICONS,
                'label_block' => false,
                'skin' => 'inline',
                'default' => [
                    'value' => 'fas fa-calendar-alt',
                    'library' => 'fa-solid',
                ],
                'condition' => [
					'bwdsplx_list_meta_active' => 'date',
                ],
            ]
        );

		$this->add_control(
            'bwdsplx_list_post_comment_icon',
            [
                'label' => esc_html__( 'Comment Icon', 'bwdsplx-post-smart' ),
                'type' => \Elementor\Controls_Manager::ICONS,
                'label_block' => false,
                'skin' => 'inline',
                'default' => [
                    'value' => 'fas fa-comments',
                    'library' => 'fa-solid',
				],
                'condition' => [
					'bwdsplx_list_meta_active' => 'comments',
                ],
            ]
		);

		$this->add_control(
			'bwdsplx_list_post_align',
			[
				'label' => __( 'Alignment', 'bwdsplx-post-smart' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::CHOOSE,
				'default' => 'top',
				'options' => [
					'top' => [
						'title' => __( 'Top', 'bwdsplx-post-smart' ),
						'icon' => 'eicon-v-align-top',
					],
					'middle' => [
						'title' => __( 'Middle', 'bwdsplx-post-smart' ),
						'icon' => 'eicon-v-align-middle',
					],
					'bottom' => [
						'title' => __( 'Bottom', 'bwdsplx-post-smart' ),
						'icon' => 'eicon-v-align-bottom',
					],
				],
				'style_transfer' => true,
				'selectors_dictionary' => [
					'top' => '-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start;',
					'middle' => '-webkit-box-align:center;-ms-flex-align:center;align-items:center;',
					'bottom' => '-webkit-box-align:end;-ms-flex-align:end;align-items:flex-end;',
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list' => '{{VALUE}};',
				],
			]
		);

		$this->end_controls_section();

	}

	//Query Settings
	protected function __query_content_controls() {

		$this->start_controls_section(
			'bwdsplx_section_spl_query',
			[
				'label' => __( 'Query', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'bwdsplx_posts_post_type',
			[
				'label' => __( 'Post Type', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::SELECT,
				'options' => Lazy_Query_Manager::get_post_types_list(),
				'default' => 'post',
			]
		);

		$this->add_control(
			'bwdsplx_posts_selected_ids',
			[
				'label' => __( 'Search & Select', 'bwdsplx-post-smart' ),
				'type' => Lazy_Select::TYPE,
				'multiple' => true,
				'label_block' => true,
				'lazy_args' => [
					'query' => Lazy_Query_Manager::QUERY_POSTS,
				],
				'condition' => [
					'bwdsplx_posts_post_type' => 'manual_selection'
				]
			]
		);

		$this->start_controls_tabs(
			'bwdsplx_tabs_posts_include_exclude',
			[
				'condition' => [
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
			]
		);
		$this->start_controls_tab(
            'bwdsplx_tab_posts_include',
            [
				'label' => __( 'Include', 'bwdsplx-post-smart' ),
				'condition' => [
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
            ]
		);

		$this->add_control(
			'bwdsplx_posts_include_by',
			[
				'label' => __( 'Include By', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::SELECT2,
				'multiple' => true,
				'label_block' => true,
				'options' => [
					'authors' => __( 'Authors', 'bwdsplx-post-smart' ),
					'terms' => __( 'Terms', 'bwdsplx-post-smart' ),
				],
				'condition' => [
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_include_author_ids',
			[
				'label' => __( 'Authors', 'bwdsplx-post-smart' ),
				'type' => Lazy_Select::TYPE,
				'multiple' => true,
				'label_block' => true,
				'lazy_args' => [
					'query' => Lazy_Query_Manager::QUERY_AUTHORS,
				],
				'condition' => [
					'bwdsplx_posts_include_by' => 'authors',
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_include_term_ids',
			[
				'label' => __( 'Terms', 'bwdsplx-post-smart' ),
				'description' => __( 'Terms are items in a taxonomy. The available taxonomies are: Categories, Tags, Formats and custom taxonomies.', 'bwdsplx-post-smart' ),
				'type' => Lazy_Select::TYPE,
				'multiple' => true,
				'label_block' => true,
				'placeholder' => __( 'Type and select terms', 'bwdsplx-post-smart' ),
				'lazy_args' => [
					'query' => Lazy_Query_Manager::QUERY_TERMS,
					'widget_props' => [
						'post_type' => 'bwdsplx_posts_post_type'
					]
				],
				'condition' => [
					'bwdsplx_posts_include_by' => 'terms',
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
			]
		);

		$this->end_controls_tab();

        $this->start_controls_tab(
            'bwdsplx_tab_posts_exclude',
            [
				'label' => __( 'Exclude', 'bwdsplx-post-smart' ),
				'condition' => [
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
            ]
		);

		$this->add_control(
			'bwdsplx_posts_exclude_by',
			[
				'label' => __( 'Exclude By', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::SELECT2,
				'multiple' => true,
				'label_block' => true,
				'options' => [
					'authors'           => __( 'Authors', 'bwdsplx-post-smart' ),
					'current_post'      => __( 'Current Post', 'bwdsplx-post-smart' ),
					'manual_selection'  => __( 'Manual Selection', 'bwdsplx-post-smart' ),
					'terms'             => __( 'Terms', 'bwdsplx-post-smart' ),
				],
				'condition' => [
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_exclude_ids',
			[
				'label' => __( 'Search & Select', 'bwdsplx-post-smart' ),
				'type' => Lazy_Select::TYPE,
				'multiple' => true,
				'label_block' => true,
				'lazy_args' => [
					'query' => Lazy_Query_Manager::QUERY_POSTS,
					'widget_props' => [
						'post_type' => 'bwdsplx_posts_post_type'
					]
				],
				'condition' => [
					'bwdsplx_posts_exclude_by' => 'manual_selection',
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_exclude_author_ids',
			[
				'label' => __( 'Authors', 'bwdsplx-post-smart' ),
				'type' => Lazy_Select::TYPE,
				'multiple' => true,
				'label_block' => true,
				'lazy_args' => [
					'query' => Lazy_Query_Manager::QUERY_AUTHORS,
				],
				'condition' => [
					'bwdsplx_posts_exclude_by' => 'authors',
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_exclude_term_ids',
			[
				'label' => __( 'Terms', 'bwdsplx-post-smart' ),
				'description' => __( 'Terms are items in a taxonomy. The available taxonomies are: Categories, Tags, Formats and custom taxonomies.', 'bwdsplx-post-smart' ),
				'type' => Lazy_Select::TYPE,
				'multiple' => true,
				'label_block' => true,
				'placeholder' => __( 'Type and select terms', 'bwdsplx-post-smart' ),
				'lazy_args' => [
					'query' => Lazy_Query_Manager::QUERY_TERMS,
					'widget_props' => [
						'post_type' => 'bwdsplx_posts_post_type'
					]
				],
				'condition' => [
					'bwdsplx_posts_exclude_by' => 'terms',
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
			]
		);

		$this->end_controls_tab();
		$this->end_controls_tabs();

		$this->add_control(
			'bwdsplx_tabs_divider',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_control(
			'bwdsplx_posts_select_date',
			[
				'label' => __( 'Date', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'anytime',
				'options' => [
					'anytime' => __( 'All', 'bwdsplx-post-smart' ),
					'today'   => __( 'Past Day', 'bwdsplx-post-smart' ),
					'week'    => __( 'Past Week', 'bwdsplx-post-smart' ),
					'month'   => __( 'Past Month', 'bwdsplx-post-smart' ),
					'quarter' => __( 'Past Quarter', 'bwdsplx-post-smart' ),
					'year'    => __( 'Past Year', 'bwdsplx-post-smart' ),
					'exact'   => __( 'Custom', 'bwdsplx-post-smart' ),
				],
				'condition' => [
					'bwdsplx_posts_post_type!' => 'manual_selection',
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_date_before',
			[
				'label' => __( 'Before', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::DATE_TIME,
				'description' => __( 'Setting a ‘Before’ date will show all the posts published until the chosen date (inclusive).', 'bwdsplx-post-smart' ),
				'condition' => [
					'bwdsplx_posts_select_date' => 'exact',
					'bwdsplx_posts_post_type!' => 'manual_selection',
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_date_after',
			[
				'label' => __( 'After', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::DATE_TIME,
				'description' => __( 'Setting an ‘After’ date will show all the posts published since the chosen date (inclusive).', 'bwdsplx-post-smart' ),
				'condition' => [
					'bwdsplx_posts_select_date' => 'exact',
					'bwdsplx_posts_post_type!' => 'manual_selection',
				]
			]
		);

		$this->add_control(
			'bwdsplx_bwdsplx_posts_orderby',
			[
				'label' => __( 'Order By', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'date',
				'options' => [
					'author'        => __( 'Author', 'bwdsplx-post-smart' ),
					'comment_count' => __( 'Comment Count', 'bwdsplx-post-smart' ),
					'date'          => __( 'Date', 'bwdsplx-post-smart' ),
					'ID'            => __( 'ID', 'bwdsplx-post-smart' ),
					'menu_order'    => __( 'Menu Order', 'bwdsplx-post-smart' ),
					'rand'          => __( 'Random', 'bwdsplx-post-smart' ),
					'title'         => __( 'Title', 'bwdsplx-post-smart' ),
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_order',
			[
				'label' => __( 'Order', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'desc',
				'options' => [
					'asc' => __( 'ASC', 'bwdsplx-post-smart' ),
					'desc' => __( 'DESC', 'bwdsplx-post-smart' ),
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_ignore_sticky_posts',
			[
				'label' => __( 'Ignore Sticky Posts', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::SWITCHER,
				'return_value' => 'yes',
				'default' => 'yes',
				'condition' => [
					'bwdsplx_posts_post_type' => 'post'
				]
			]
		);

		$this->add_control(
			'bwdsplx_posts_only_with_featured_image',
			[
				'label' => __( 'With Featured Image', 'bwdsplx-post-smart' ),
				'description' => __( 'Enable to display posts only when featured image is set.', 'bwdsplx-post-smart' ),
				'type' => Controls_Manager::SWITCHER,
				'return_value' => 'yes',
				'condition' => [
					'bwdsplx_posts_post_type!' => 'manual_selection'
				]
			]
		);

		$this->end_controls_section();
	}

	//Layout Style
	protected function __layout_style_controls(){

		$this->start_controls_section(
			'bwdsplx_section_spl_layout_style',
			[
				'label' => __( 'Layout', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				// 'condition' => [
				// 	'bwdsplx_make_featured_post' => 'yes',
				// 	'bwdsplx_show_badge' => 'yes',
				// ],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_grid_gap',
			[
				'label' => __( 'Column Gap', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 200,
						'step' => 1,
					]
				],
				'default' => [
					'unit' => 'px',
					'size' => '30',
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-grid-area' => 'grid-column-gap: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .bwdsplx-spl-grid-area.bwdsplx-spl-featured-post-on .bwdsplx-spl-list-wrap' => 'grid-column-gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_grid_row_gap',
			[
				'label' => __( 'Row Gap', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 200,
						'step' => 1,
					]
				],
				'default' => [
					'unit' => 'px',
					'size' => '30',
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-grid-area' => 'grid-row-gap: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .bwdsplx-spl-grid-area.bwdsplx-spl-featured-post-on .bwdsplx-spl-list-wrap' => 'grid-row-gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_single_grid_height',
			[
				'label' => __( 'Feature Item Height', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 1,
					]
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-grid-area.bwdsplx-spl-col-1.bwdsplx-spl-featured-post-on' => 'grid-template-rows: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .bwdsplx-spl-grid-area.bwdsplx-spl-col-2.bwdsplx-spl-featured-post-on' => 'grid-template-rows: {{SIZE}}{{UNIT}};',
				],
				'conditions' => $this->conditions('feature_item_height'),
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_post_list_grid_height',
			[
				'label' => __( 'Post List Item Height', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 1,
					]
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-grid-area.bwdsplx-spl-featured-post-off' => 'grid-auto-rows: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .bwdsplx-spl-list-wrap' => 'grid-auto-rows: {{SIZE}}{{UNIT}};',
				],
				'conditions' => $this->conditions('list_post_show'),
			]
		);

		$this->end_controls_section(); //Layout Style End
	}

	//Top Bar Style
	protected function __top_bar_style_controls() {

		//Top Bar Style
		$this->start_controls_section(
			'bwdsplx_section_spl_top_bar_style',
			[
				'label' => __( 'Topbar', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'bwdsplx_top_bar_show' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_margin',
			[
				'label' => __( 'Margin', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-header' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_padding',
			[
				'label' => __( 'Padding', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-header' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				]
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'bwdsplx_spl_top_bar_background',
				'label' => __( 'Background', 'bwdsplx-post-smart' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .bwdsplx-spl-header',
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'bwdsplx_spl_top_bar_box_shadow',
				'label' => __( 'Box Shadow', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-header',
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_top_bar_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-header',
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_border_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-header' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section(); //Top Bar Style End
	}

	//Top Bar Title Style
	protected function __top_bar_title_style_controls(){

		$this->start_controls_section(
			'bwdsplx_section_spl_top_bar_title_style',
			[
				'label' => __( 'Topbar Title', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'bwdsplx_top_bar_show' => 'yes',
					'bwdsplx_widget_title!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_widget_title_margin_right',
			[
				'label' => __( 'Margin Right', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 200,
						'step' => 1,
					],
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-widget-title' => 'margin-right: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_widget_title!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_widget_title_padding',
			[
				'label' => __( 'Padding', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-widget-title' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_widget_title!' => '',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'bwdsplx_spl_top_bar_widget_title_background',
				'label' => __( 'Background', 'bwdsplx-post-smart' ),
				'types' => [ 'classic', 'gradient' ],
				'exclude' => [
					'image'
				],
				'selector' => '{{WRAPPER}} .bwdsplx-spl-widget-title',
				'condition' => [
					'bwdsplx_widget_title!' => '',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_top_bar_widget_title_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-widget-title',
				'condition' => [
					'bwdsplx_widget_title!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_widget_title_border_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-widget-title' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_widget_title!' => '',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'spl_top_bar_bwdsplx_widget_title_typography',
				'selector' => '{{WRAPPER}} .bwdsplx-spl-widget-title',
				'condition' => [
					'bwdsplx_widget_title!' => '',
				],
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_widget_title_color',
			[
				'label' => __( 'Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-widget-title' => 'color: {{VALUE}}',
				],
				'condition' => [
					'bwdsplx_widget_title!' => '',
				],
			]
		);

		$this->end_controls_section(); //Top Bar Title Style End
	}

	//Top Bar Filter Style
	protected function __top_bar_filter_style_controls(){

		$this->start_controls_section(
			'bwdsplx_section_spl_top_bar_filter_style',
			[
				'label' => __( 'Topbar Filter', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'bwdsplx_top_bar_show' => 'yes',
					'bwdsplx_category_filter' => 'yes',
				],
			]
		);

		//Inline Filter
		$this->add_control(
			'bwdsplx_spl_top_bar_filter_heading',
			[
				'label' => __( 'Inline Filter', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
					'bwdsplx_category_filter_style' => 'inline',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_filter_item_margin',
			[
				'label' => __( 'Item margin', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
					'bwdsplx_category_filter_style' => 'inline',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_filter_item_padding',
			[
				'label' => __( 'Item Padding', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
					'bwdsplx_category_filter_style' => 'inline',
				]
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_top_bar_filter_item_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span',
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
					'bwdsplx_category_filter_style' => 'inline',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_filter_item_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
					'bwdsplx_category_filter_style' => 'inline',
				]
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'spl_top_bar_filter_item_typography',
				'selector' => '{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span',
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
					'bwdsplx_category_filter_style' => 'inline',
				],
			]
		);

		$this->start_controls_tabs( 'bwdsplx_spl_top_bar_filter_tabs',
			[
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
					'bwdsplx_category_filter_style' => 'inline',
				]
			]
		);
		$this->start_controls_tab(
			'bwdsplx_spl_top_bar_filter_normal_tab',
			[
				'label' => __( 'Normal', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_filter_item_color',
			[
				'label' => __( 'Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span' => 'color: {{VALUE}}',
				]
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_filter_item_background_color',
			[
				'label' => __( 'Background Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span' => 'background-color: {{VALUE}}',
				]
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'bwdsplx_spl_top_bar_filter_hover_tab',
			[
				'label' => __( 'Hover/Active', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_filter_item_hover_color',
			[
				'label' => __( 'Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span:hover' => 'color: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span.bwdsplx-active' => 'color: {{VALUE}}',
				]
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_filter_item_hover_background_color',
			[
				'label' => __( 'Background Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span:hover' => 'background-color: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-filter ul.bwdsplx-spl-filter-list li span.bwdsplx-active' => 'background-color: {{VALUE}}',
				]
			]
		);

		$this->end_controls_tab();
		$this->end_controls_tabs();

		//Dropdown Filter
		$this->add_control(
			'bwdsplx_spl_top_bar_select_heading',
			[
				'label' => __( 'Dropdown Filter', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_nice_select_height',
			[
				'label' => __( 'Select Box Height', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select' => 'height: {{SIZE}}{{UNIT}}; line-height: {{SIZE}}{{UNIT}};'
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_nice_select_space',
			[
				'label' => __( 'Space Right', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select' => 'margin-right: {{SIZE}}{{UNIT}};'
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_nice_select_padding',
			[
				'label' => __( 'Dropdown Item Padding', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .option' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_top_bar_nice_select_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select,{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .list',
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_nice_select_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .list' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'spl_top_bar_nice_select_typography',
				'selector' => '{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select span.current, 
				{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .option',
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				],
			]
		);

		$this->start_controls_tabs( 'bwdsplx_spl_top_bar_nice_select_tabs',
			[
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);
		$this->start_controls_tab(
			'bwdsplx_spl_top_bar_nice_select_normal_tab',
			[
				'label' => __( 'Normal', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nice_select_color',
			[
				'label' => __( 'Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select span.current' => 'color: {{VALUE}}',
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select:after' => 'border-color: {{VALUE}}',
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .option' => 'color: {{VALUE}}',
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nice_select_background_color',
			[
				'label' => __( 'Background Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select' => 'background-color: {{VALUE}}',
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .option' => 'background-color: {{VALUE}}',
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .list:hover .option:not(:hover)' => 'background-color: {{VALUE}}!important',
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);


		$this->end_controls_tab();

		$this->start_controls_tab(
			'bwdsplx_spl_top_bar_nice_select_hover_tab',
			[
				'label' => __( 'Hover/Active', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nice_select_hover_color',
			[
				'label' => __( 'Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .option:hover' => 'color: {{VALUE}}',
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .option.focus' => 'color: {{VALUE}}',
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .option.selected.focus' => 'color: {{VALUE}}',
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nice_select_hover_background_color',
			[
				'label' => __( 'Background Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .option:hover' => 'background-color: {{VALUE}}',
					'{{WRAPPER}} .nice-select.bwdsplx-spl-custom-select .option.selected.focus' => 'background-color: {{VALUE}}',
				],
				'condition' => [
					'bwdsplx_category_filter' => 'yes',
				]
			]
		);

		$this->end_controls_tab();
		$this->end_controls_tabs();


		$this->end_controls_section(); //Top Bar Filter Style End
	}

	//Top Bar Navigation Style
	protected function __top_bar_navigation_style_controls(){

		$this->start_controls_section(
			'bwdsplx_section_spl_top_bar_navigation_style',
			[
				'label' => __( 'Topbar Navigation', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'bwdsplx_top_bar_show' => 'yes',
					'bwdsplx_navigation_show' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_nav_space',
			[
				'label' => __( 'Space Between', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button:first-child' => 'margin-right: {{SIZE}}{{UNIT}};'
				],
				'condition' => [
					'bwdsplx_navigation_show' => 'yes',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_nav_padding',
			[
				'label' => __( 'Padding', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_navigation_show' => 'yes',
				]
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_top_bar_nav_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-pagination button',
				'exclude' => ['color'],
				'condition' => [
					'bwdsplx_navigation_show' => 'yes',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_nav_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_navigation_show' => 'yes',
				]
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_top_bar_nav_font_size',
			[
				'label' => __( 'Icon Size', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button i' => 'font-size: {{SIZE}}{{UNIT}};'
				],
				'condition' => [
					'bwdsplx_navigation_show' => 'yes',
				]
			]
		);

		$this->start_controls_tabs( 'bwdsplx_spl_top_bar_nav_tabs',
			[
				'condition' => [
					'bwdsplx_navigation_show' => 'yes',
				]
			]
		);
		$this->start_controls_tab(
			'bwdsplx_spl_top_bar_nav_normal_tab',
			[
				'label' => __( 'Normal', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nav_color',
			[
				'label' => __( 'Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nav_background_color',
			[
				'label' => __( 'Background Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button' => 'background-color: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nav_border_color',
			[
				'label' => __( 'Border Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button' => 'border-color: {{VALUE}}',
				],
			]
		);


		$this->end_controls_tab();

		$this->start_controls_tab(
			'bwdsplx_spl_top_bar_nav_hover_tab',
			[
				'label' => __( 'Hover', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nav_hover_color',
			[
				'label' => __( 'Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nav_hover_background_color',
			[
				'label' => __( 'Background Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button:hover' => 'background-color: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'bwdsplx_spl_top_bar_nav_border_hover_color',
			[
				'label' => __( 'Border Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-pagination button:hover' => 'border-color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();
		$this->end_controls_tabs();


		$this->end_controls_section(); //Top Bar Navigation Style End
	}

	//Featured Post Style
	protected function __bwdsplx_featured_post_style_controls(){

		//Feature Post Style
		$this->start_controls_section(
			'bwdsplx_section_spl_feature_post_style',
			[
				'label' => __( 'Featured Post', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_padding',
			[
				'label' => __( 'Padding', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post-wrap' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				]
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'bwdsplx_spl_feature_post_background',
				'label' => __( 'Background', 'bwdsplx-post-smart' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post-wrap',
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'bwdsplx_spl_feature_post_box_shadow',
				'label' => __( 'Box Shadow', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post-wrap',
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_feature_post_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post-wrap',
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_border_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post-wrap' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		//Feature Post Image overlay color
		$this->add_control(
			'bwdsplx_spl_feature_post_image_overlay_heading',
			[
				'label' => __( 'Image Overlay', 'bwdsplx-post-smart' ),
				'description' => __( 'This overlay color only apply when post has an image.', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					'bwdsplx_featured_post_style' => 'inside-conent',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'bwdsplx_spl_feature_post_image_overlay',
				'label' => __( 'Background', 'bwdsplx-post-smart' ),
				'description' => __( 'This overlay color only apply when post has an image.', 'bwdsplx-post-smart' ),
				'types' => [ 'classic', 'gradient' ],
				'exclude' => [
					'image'
				],
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-inside-conent .bwdsplx-spl-featured-thumb:before',
				'condition' => [
					'bwdsplx_featured_post_style' => 'inside-conent',
				],
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_post_image_overlay_note',
			[
				'label' => __( 'Image Overlay Note', 'bwdsplx-post-smart' ),
				'show_label' => false,
				'type' => \Elementor\Controls_Manager::RAW_HTML,
				'raw' => __( 'This overlay color only apply when post has an image.', 'bwdsplx-post-smart' ),
				'content_classes' => 'elementor-control-field-description',
				'condition' => [
					'bwdsplx_featured_post_style' => 'inside-conent',
				],
			]
		);

		//Feature Post Image
		$this->add_control(
			'bwdsplx_spl_feature_post_image_heading',
			[
				'label' => __( 'Image', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					'bwdsplx_featured_post_style' => 'outside-conent',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_image_width',
			[
				'label' => __( 'Width', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px','%' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 1,
					],
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-outside-conent .bwdsplx-spl-featured-thumb' => 'width: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_featured_post_style' => 'outside-conent',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_image_margin_btm',
			[
				'label' => __( 'Margin Bottom', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-outside-conent .bwdsplx-spl-featured-thumb' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_featured_post_style' => 'outside-conent',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'bwdsplx_spl_feature_post_image_shadow',
				'label' => __( 'Box Shadow', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-outside-conent .bwdsplx-spl-featured-thumb img',
				'condition' => [
					'bwdsplx_featured_post_style' => 'outside-conent',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_feature_post_image_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-outside-conent .bwdsplx-spl-featured-thumb img',
				'condition' => [
					'bwdsplx_featured_post_style' => 'outside-conent',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_image_border_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-outside-conent .bwdsplx-spl-featured-thumb img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_featured_post_style' => 'outside-conent',
				],
			]
		);

		//Feature Post Title
		$this->add_control(
			'bwdsplx_spl_feature_post_title_heading',
			[
				'label' => __( 'Title', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					'bwdsplx_featured_post_title' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_title_margin_btm',
			[
				'label' => __( 'Margin Bottom', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 200,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => '10',
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-title' => 'margin-bottom: {{SIZE}}{{UNIT}}; margin-top: 0;',
				],
				'condition' => [
					'bwdsplx_featured_post_title' => 'yes',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'spl_feature_post_title_typography',
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-title a',
				'condition' => [
					'bwdsplx_featured_post_title' => 'yes',
				],
			]
		);

		$this->start_controls_tabs( 'bwdsplx_spl_feature_post_title_tabs',
			[
				'condition' => [
					'bwdsplx_featured_post_title' => 'yes',
				],
			]
		);
		$this->start_controls_tab(
			'bwdsplx_spl_feature_post_title_normal_tab',
			[
				'label' => __( 'Normal', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_post_title_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-title a' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'bwdsplx_spl_feature_post_title_hover_tab',
			[
				'label' => __( 'Hover', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_post_title_hover_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-title a:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();
		$this->end_controls_tabs();

		//Feature Post Meta
		$this->add_control(
			'bwdsplx_spl_feature_post_meta_heading',
			[
				'label' => __( 'Meta', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
				'conditions' => $this->conditions('feature_meta_style'),
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_meta_icon_size',
			[
				'label' => __( 'Icon Size', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta .bwdsplx-spl-meta-icon' => 'font-size: {{SIZE}}{{UNIT}};',
				],
				'conditions' => $this->conditions('feature_meta_style'),
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_meta_icon_space',
			[
				'label' => __( 'Icon Space', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta ul li i' => 'margin-right: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta ul li svg' => 'margin-right: {{SIZE}}{{UNIT}};',
				],
				'conditions' => $this->conditions('feature_meta_style'),
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_meta_space',
			[
				'label' => __( 'Space Between', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta ul li' => 'margin-right: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta ul li:last-child' => 'margin-right: 0;',
				],
				'conditions' => $this->conditions('feature_meta_style'),
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_post_meta_margin_btm',
			[
				'label' => __( 'Margin Bottom', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'default' => [
					'unit' => 'px',
					'size' => 15,
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta ul li' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
				'conditions' => $this->conditions('feature_meta_style'),
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'spl_feature_post_meta_typography',
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta .bwdsplx-spl-meta-text',
				'conditions' => $this->conditions('feature_meta_style'),
			]
		);

		$this->start_controls_tabs( 'bwdsplx_spl_feature_post_meta_tabs',
			[
				'conditions' => $this->conditions('feature_meta_style'),
			]
		);
		$this->start_controls_tab(
			'bwdsplx_spl_feature_post_meta_normal_tab',
			[
				'label' => __( 'Normal', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_post_meta_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta .bwdsplx-spl-meta-icon i' => 'color: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta .bwdsplx-spl-meta-icon path' => 'fill: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta .bwdsplx-spl-meta-text' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'bwdsplx_spl_feature_post_meta_hover_tab',
			[
				'label' => __( 'Hover', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_post_meta_hover_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta a:hover .bwdsplx-spl-meta-icon i' => 'color: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta a:hover .bwdsplx-spl-meta-icon path' => 'fill: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-meta a:hover .bwdsplx-spl-meta-text' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();
		$this->end_controls_tabs();

		//Feature Post Content
		$this->add_control(
			'bwdsplx_spl_feature_post_content_heading',
			[
				'label' => __( 'Content', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					'bwdsplx_featured_excerpt_length!' => '',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'spl_feature_post_content_typography',
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-desc',
				'condition' => [
					'bwdsplx_featured_excerpt_length!' => '',
				],
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_post_content_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-desc' => 'color: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-desc p' => 'margin-bottom: 0',
				],
				'condition' => [
					'bwdsplx_featured_excerpt_length!' => '',
				],
			]
		);

		$this->end_controls_section(); //Feature Post Style End
	}

	//Featured Post Badge Style
	protected function __featured_post_badge_style_controls(){

		//Taxonomy Badge
		$this->start_controls_section(
			'bwdsplx_section_spl_featured_post_badge_style',
			[
				'label' => __( 'Badge', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'bwdsplx_make_featured_post' => 'yes',
					'bwdsplx_show_badge' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_badge_margin_btm',
			[
				'label' => __( 'Margin Bottom', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 200,
						'step' => 1,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_show_badge' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_badge_padding',
			[
				'label' => __( 'Padding', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_show_badge' => 'yes',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_feature_badge_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'exclude' => [
					'color'
				],
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a',
				'condition' => [
					'bwdsplx_show_badge' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_feature_badge_border_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_show_badge' => 'yes',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'spl_feature_badge_typography',
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a',
				'condition' => [
					'bwdsplx_show_badge' => 'yes',
				],
			]
		);

		$this->start_controls_tabs( 'bwdsplx_spl_feature_badge_tabs',
			[
				'condition' => [
					'bwdsplx_show_badge' => 'yes',
				],
			]
		);
		$this->start_controls_tab(
			'bwdsplx_spl_feature_badge_normal_tab',
			[
				'label' => __( 'Normal', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_badge_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'bwdsplx_spl_feature_badge_background',
				'label' => __( 'Background', 'bwdsplx-post-smart' ),
				'types' => [ 'classic', 'gradient' ],
				'exclude' => [
					'image'
				],
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a',
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_badge_border_color',
			[
				'label' => __( 'Border Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a' => 'border-color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'bwdsplx_spl_feature_badge_hover_tab',
			[
				'label' => __( 'Hover', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_badge_hover_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'bwdsplx_spl_feature_badge_hover_background',
				'label' => __( 'Background', 'bwdsplx-post-smart' ),
				'types' => [ 'classic', 'gradient' ],
				'exclude' => [
					'image'
				],
				'selector' => '{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a:hover',
			]
		);

		$this->add_control(
			'bwdsplx_spl_feature_badge_hover_border_color',
			[
				'label' => __( 'Border Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-featured-post .bwdsplx-spl-badge a:hover' => 'border-color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();
		$this->end_controls_tabs();

		$this->end_controls_section(); //Featured Post Badge Style End
	}

	//List Post Style
	protected function __list_post_style_controls(){

		//List Post Style
		$this->start_controls_section(
			'bwdsplx_section_spl_list_post_style',
			[
				'label' => __( 'List Post', 'bwdsplx-post-smart' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'conditions' => $this->conditions('list_post_show'),
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_list_post_padding',
			[
				'label' => __( 'Padding', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				]
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'bwdsplx_spl_list_post_background',
				'label' => __( 'Background', 'bwdsplx-post-smart' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .bwdsplx-spl-list',
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'bwdsplx_spl_list_post_box_shadow',
				'label' => __( 'Box Shadow', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-list',
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_list_post_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-list',
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_list_post_border_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		//List Post Image
		$this->add_control(
			'bwdsplx_spl_bwdsplx_list_post_image_heading',
			[
				'label' => __( 'Image', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					'bwdsplx_list_post_image' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_bwdsplx_list_post_image_width',
			[
				'label' => __( 'Width', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px','%' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 1,
					],
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-list-thumb' => 'max-width: {{SIZE}}{{UNIT}};-webkit-box-flex: 0;-webkit-flex: 0 0 {{SIZE}}{{UNIT}};-ms-flex: 0 0 {{SIZE}}{{UNIT}};flex: 0 0 {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_list_post_image' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_bwdsplx_list_post_image_margin_right',
			[
				'label' => __( 'Margin Right', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-list-thumb' => 'margin-right: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_list_post_image' => 'yes',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'bwdsplx_spl_bwdsplx_list_post_image_shadow',
				'label' => __( 'Box Shadow', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-list-thumb img',
				'condition' => [
					'bwdsplx_list_post_image' => 'yes',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'bwdsplx_spl_bwdsplx_list_post_image_border',
				'label' => __( 'Border', 'bwdsplx-post-smart' ),
				'selector' => '{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-list-thumb img',
				'condition' => [
					'bwdsplx_list_post_image' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_bwdsplx_list_post_image_border_radius',
			[
				'label' => __( 'Border Radius', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', 'em', '%' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-list-thumb img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [
					'bwdsplx_list_post_image' => 'yes',
				],
			]
		);

		//List Post Title
		$this->add_control(
			'bwdsplx_spl_list_post_title_heading',
			[
				'label' => __( 'Title', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_list_post_title_margin_btm',
			[
				'label' => __( 'Margin Bottom', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 200,
						'step' => 1,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-list-title' => 'margin-bottom: {{SIZE}}{{UNIT}}; margin-top: 0;',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'spl_list_post_title_typography',
				'selector' => '{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-list-title a',
			]
		);

		$this->start_controls_tabs( 'bwdsplx_spl_list_post_title_tabs');
		$this->start_controls_tab(
			'bwdsplx_spl_list_post_title_normal_tab',
			[
				'label' => __( 'Normal', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_list_post_title_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-list-title a' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'bwdsplx_spl_list_post_title_hover_tab',
			[
				'label' => __( 'Hover', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_list_post_title_hover_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-list-title a:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();
		$this->end_controls_tabs();

		//List Post Meta
		$this->add_control(
			'bwdsplx_spl_list_post_meta_heading',
			[
				'label' => __( 'Meta', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
				'conditions' => $this->conditions('list_post_meta_style'),
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_list_post_meta_icon_size',
			[
				'label' => __( 'Icon Size', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta .bwdsplx-spl-meta-icon' => 'font-size: {{SIZE}}{{UNIT}};',
				],
				'conditions' => $this->conditions('list_post_meta_style'),
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_list_post_meta_icon_space',
			[
				'label' => __( 'Icon Space', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta ul li i' => 'margin-right: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta ul li svg' => 'margin-right: {{SIZE}}{{UNIT}};',
				],
				'conditions' => $this->conditions('list_post_meta_style'),
			]
		);

		$this->add_responsive_control(
			'bwdsplx_spl_list_post_meta_space',
			[
				'label' => __( 'Space Between', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta ul li' => 'margin-right: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta ul li:last-child' => 'margin-right: 0;',
				],
				'conditions' => $this->conditions('list_post_meta_style'),
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'spl_list_post_meta_typography',
				'selector' => '{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta .bwdsplx-spl-meta-text',
				'conditions' => $this->conditions('list_post_meta_style'),
			]
		);

		$this->start_controls_tabs( 'bwdsplx_spl_list_post_meta_tabs',
			[
				'conditions' => $this->conditions('list_post_meta_style'),
			]
		);
		$this->start_controls_tab(
			'bwdsplx_spl_list_post_meta_normal_tab',
			[
				'label' => __( 'Normal', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_list_post_meta_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta .bwdsplx-spl-meta-icon i' => 'color: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta .bwdsplx-spl-meta-icon path' => 'fill: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta .bwdsplx-spl-meta-text' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'bwdsplx_spl_list_post_meta_hover_tab',
			[
				'label' => __( 'Hover', 'bwdsplx-post-smart' ),
			]
		);

		$this->add_control(
			'bwdsplx_spl_list_post_meta_hover_color',
			[
				'label' => __( 'Text Color', 'bwdsplx-post-smart' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta a:hover .bwdsplx-spl-meta-icon i' => 'color: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta a:hover .bwdsplx-spl-meta-icon path' => 'fill: {{VALUE}}',
					'{{WRAPPER}} .bwdsplx-spl-list .bwdsplx-spl-meta a:hover .bwdsplx-spl-meta-text' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();
		$this->end_controls_tabs();

		$this->end_controls_section(); //List Post Style End
	}

	/**
	 * get column class
	 */
	public function get_column_cls ($column) {

		switch( $column ){
			case "col-1":
				$column_cls = "bwdsplx-spl-col-1";
				break;
			case "col-2":
				$column_cls = "bwdsplx-spl-col-2";
				break;
			case "col-3":
				$column_cls = "bwdsplx-spl-col-3";
				break;
			default:
			  	$column_cls = "bwdsplx-spl-col-3";
		}

		return $column_cls;

	}

	/**
	 * get featured column class
	 */
	public function get_featured_column_cls ($column) {

		switch( $column ){
			case "featured-col-1":
				$column_cls = "bwdsplx-spl-featured-col-1";
				break;
			case "featured-col-2":
				$column_cls = "bwdsplx-spl-featured-col-2";
				break;
			default:
			  	$column_cls = "bwdsplx-spl-featured-col-1";
		}

		return $column_cls;

	}

	/**
	 * get item per page
	 */
	public function get_item_per_page ($item = 'per_page') {
		$settings = $this->get_settings_for_display();

		$per_page = -1;
		$list_column = '';

		if( 'col-1' === $settings['column'] ) {
			if( 'yes' === $settings['bwdsplx_make_featured_post'] ){
				$per_page = 1;
			}else{
				$per_page = 4;
				$list_column = 'bwdsplx-spl-list-col-1';
			}
		}

		if( 'col-2' === $settings['column'] ) {
			if( 'yes' === $settings['bwdsplx_make_featured_post'] && 'featured-col-1' === $settings['bwdsplx_featured_post_column'] ){
				$per_page = 5;
				$list_column = 'bwdsplx-spl-list-col-1';
			}elseif( 'yes' === $settings['bwdsplx_make_featured_post'] && 'featured-col-2' === $settings['bwdsplx_featured_post_column'] ){
				$per_page = 1;
			}else{
				$per_page = 8;
				$list_column = 'bwdsplx-spl-list-col-2';
			}
		}

		if( 'col-3' === $settings['column'] ) {
			if( 'yes' === $settings['bwdsplx_make_featured_post'] && 'featured-col-1' === $settings['bwdsplx_featured_post_column'] ){
				$per_page = 9;
				$list_column = 'bwdsplx-spl-list-col-2';
			}elseif( 'yes' === $settings['bwdsplx_make_featured_post'] && 'featured-col-2' === $settings['bwdsplx_featured_post_column'] ){
				$per_page = 5;
				$list_column = 'bwdsplx-spl-list-col-1';
			}else{
				$per_page = 12;
				$list_column = 'bwdsplx-spl-list-col-3';
			}
		}

		if( $item === 'list_column' ){
			return $list_column;
		}else{
			return $per_page;
		}

	}

	/**
	 * render header markup
	 */
	public function render_header_markup ($header = 'yes') {
		if ( 'yes' != $header ) {
			return;
		}
		$settings = $this->get_settings_for_display();

		$post_taxonomies = get_object_taxonomies(  $settings['bwdsplx_posts_post_type'] );
		if( !empty( $settings['bwdsplx_filter_terms_ids'] ) ){
			$categories = get_terms( [
				'taxonomy' => $post_taxonomies,
				'term_taxonomy_id' => $settings['bwdsplx_filter_terms_ids'],
				// 'include' => $settings['bwdsplx_filter_terms_ids'],
				'hide_empty' => false,
				// 'orderby' => 'include',
			] );
		}else{
			$categories = get_terms( [
				'taxonomy' => $post_taxonomies,
				'hide_empty' => false,
			] );
		}

		// echo '<pre>';
		// var_dump( $post_taxonomies);
		// echo '</pre>';

		?>
			<!-- header -->
			<div class="bwdsplx-spl-header">
				<?php
				if (  $settings['bwdsplx_widget_title'] ) {
					printf( '<%1$s %2$s>%3$s</%1$s>',
						bwdsplx_escape_tags( $settings['bwdsplx_widget_title_tag'] ),
						'class="bwdsplx-spl-widget-title"',
						esc_html( $settings['bwdsplx_widget_title'] )
					);
				}
				?>
				<?php if ( 'yes' === $settings['bwdsplx_category_filter'] ): ?>
					<div class="bwdsplx-spl-filter <?php echo esc_attr($settings['bwdsplx_category_filter_style']);?>">
						<?php if ( 'inline' === $settings['bwdsplx_category_filter_style'] ): ?>
						<ul class="bwdsplx-spl-filter-list">
							<?php if( $settings['bwdsplx_filter_all_text'] ):?>
								<li><span class="bwdsplx-active" data-value="<?php echo esc_html( $settings['bwdsplx_filter_all_text'] );?>"><?php echo esc_html( $settings['bwdsplx_filter_all_text'] );?></span></li>
							<?php endif; ?>
							<?php foreach ( $categories as $category ): ?>
								<li><span data-value="<?php echo esc_attr($category->term_taxonomy_id);?>"><?php echo esc_html( $category->name );?></span></li>
							<?php endforeach; ?>
						</ul>
						<?php endif; ?>
						<select class="bwdsplx-spl-custom-select">
							<?php if( $settings['bwdsplx_filter_all_text'] ):?>
								<option data-display="<?php echo esc_html( $settings['bwdsplx_filter_all_text'] );?>"><?php echo esc_html( $settings['bwdsplx_filter_all_text'] );?></option>
							<?php endif; ?>
							<?php foreach ( $categories as $category ): ?>
								<option value="<?php echo esc_attr($category->term_taxonomy_id);?>"><?php echo esc_html( $category->name );?></option>
							<?php endforeach; ?>
						</select>
					</div>
				<?php endif; ?>
				<?php if ( 'yes' === $settings['bwdsplx_navigation_show'] ): ?>
					<div class="bwdsplx-spl-pagination">
						<button class="prev"><i class="fa fa-angle-left"></i></button>
						<button class="next"><i class="fa fa-angle-right"></i></button>
					</div>
				<?php endif; ?>
			</div>
			<!-- /header -->
		<?php
	}

	protected $query_args = [];
	protected function get_query_args() {
		$this->setup_query_base();
		$this->setup_query_post_type();
		$this->setup_query_ignore_sticky();
		$this->setup_query_exclude_post();
		$this->setup_query_authors();
		$this->setup_query_taxonomy();
		$this->setup_query_with_featured_image();
		$this->setup_query_date();

		$this->setup_query_include_manual();

		$this->setup_query_order();

		return $this->query_args;
	}

	protected function setup_query_base() {
		$this->query_args['post_status'] = 'publish';
		$this->query_args['suppress_filters'] = false;
	}

	protected function get_query_post_type() {
		return $this->get_settings_for_display( 'bwdsplx_posts_post_type' );
	}

	protected function is_query_manual() {
		return ( $this->get_query_post_type() === 'manual_selection' );
	}

	protected function setup_query_post_type() {
		if ( ! $this->is_query_manual() ) {
			$this->query_args['post_type'] = $this->get_query_post_type();
		} elseif ( $this->is_query_manual() ) {
			$this->query_args['post_type'] = 'any';
		}
	}

	protected function setup_query_date() {
		if ( $this->is_query_manual() ) {
			return;
		}

		$settings = $this->get_settings_for_display();
		$selected_date = $settings['bwdsplx_posts_select_date'];

		if ( ! empty( $selected_date ) ) {
			$date_query = [];

			switch ( $selected_date ) {
				case 'today':
					$date_query['after'] = '-1 day';
					break;

				case 'week':
					$date_query['after'] = '-1 week';
					break;

				case 'month':
					$date_query['after'] = '-1 month';
					break;

				case 'quarter':
					$date_query['after'] = '-3 month';
					break;

				case 'year':
					$date_query['after'] = '-1 year';
					break;

				case 'exact':
					$after_date = $settings['bwdsplx_posts_date_after'];
					if ( ! empty( $after_date ) ) {
						$date_query['after'] = $after_date;
					}

					$before_date = $settings['bwdsplx_posts_date_before'];
					if ( ! empty( $before_date ) ) {
						$date_query['before'] = $before_date;
					}

					$date_query['inclusive'] = true;
					break;
			}

			if ( ! empty( $date_query ) ) {
				$this->query_args['date_query'] = $date_query;
			}
		}
	}

	protected function setup_query_include_manual() {
		if ( ! $this->is_query_manual() ) {
			return;
		}

		$selected_ids = $this->get_settings_for_display( 'bwdsplx_posts_selected_ids' );
		$selected_ids = wp_parse_id_list( $selected_ids );

		if ( ! empty( $selected_ids ) ) {
			$this->query_args['post__in'] = $selected_ids;
		}
	}

	protected function setup_query_ignore_sticky() {
		if ( $this->get_query_post_type() === 'post' &&
			$this->get_settings_for_display( 'bwdsplx_posts_ignore_sticky_posts' ) === 'yes'
			) {
			$this->query_args['ignore_sticky_posts'] = true;
		}
	}

	protected function setup_query_with_featured_image() {
		if ( $this->is_query_manual() ) {
			return;
		}

		if ( $this->get_settings_for_display( 'bwdsplx_posts_only_with_featured_image' ) === 'yes' ) {
			$this->query_args['meta_key'] = '_thumbnail_id';
		}
	}

	protected function setup_query_order() {
		$this->query_args['order'] = $this->get_settings_for_display( 'bwdsplx_posts_order' );
		$this->query_args['orderby'] = $this->get_settings_for_display( 'bwdsplx_bwdsplx_posts_orderby' );
	}

	protected function get_query_param_by( $by = 'exclude' ) {
		$by_map = [
			'exclude' => 'bwdsplx_posts_exclude_by',
			'include' => 'bwdsplx_posts_include_by',
		];

		$_setting = $this->get_settings_for_display( $by_map[ $by ] );

		return ( ! empty( $_setting ) ? $_setting : [] );
	}

	protected function setup_query_exclude_post() {
		if ( $this->is_query_manual() ) {
			return;
		}

		$settings = $this->get_settings_for_display();
		$exclude_by = $this->get_query_param_by( 'exclude' );
		$current_post = [];

		if ( in_array( 'current_post', $exclude_by ) && is_singular() ) {
			$current_post = [ get_the_ID() ];
		}
		
		if ( in_array( 'current_post', $exclude_by ) ) {
			$posts = get_posts();
			$current_post = !empty($posts) && is_array($posts) ? [ $posts[0]->ID ] : [];
		}

		if ( in_array( 'manual_selection', $exclude_by ) || in_array( 'current_post', $exclude_by ) ) {
			$exclude_ids = $settings['bwdsplx_posts_exclude_ids'];
			$this->query_args['post__not_in'] = array_merge( $current_post, wp_parse_id_list( $exclude_ids ) );
		}
	}

	protected function setup_query_taxonomy() {
		if ( $this->is_query_manual() ) {
			return;
		}

		$settings = $this->get_settings_for_display();
		$include_by = $this->get_query_param_by( 'include' );
		$exclude_by = $this->get_query_param_by( 'exclude' );
		$include_terms = [];
		$exclude_terms = [];
		$terms_query = [];

		if ( in_array( 'terms', $include_by ) ) {
			$include_terms = wp_parse_id_list( $settings['bwdsplx_posts_include_term_ids'] );
		}

		if ( in_array( 'terms', $exclude_by ) ) {
			$exclude_terms = wp_parse_id_list( $settings['bwdsplx_posts_exclude_term_ids'] );
			$include_terms = array_diff( $include_terms, $exclude_terms );
		}

		if ( ! empty( $include_terms ) ) {
			$tax_terms_map = self::get_query_tax_terms_map( $include_terms );

			foreach ( $tax_terms_map as $tax => $terms ) {
				$terms_query[] = [
					'taxonomy' => $tax,
					'field'    => 'term_id',
					'terms'    => $terms,
					'operator' => 'IN',
				];
			}
		}

		if ( ! empty( $exclude_terms ) ) {
			$tax_terms_map = self::get_query_tax_terms_map( $exclude_terms );

			foreach ( $tax_terms_map as $tax => $terms ) {
				$terms_query[] = [
					'taxonomy' => $tax,
					'field'    => 'term_id',
					'terms'    => $terms,
					'operator' => 'NOT IN',
				];
			}
		}

		if ( ! empty( $terms_query ) ) {
			$this->query_args['tax_query'] = $terms_query;
			$this->query_args['tax_query']['relation'] = 'AND';
		}
	}

	protected static function get_query_tax_terms_map( $term_tax_ids = [] ) {
		$terms = get_terms( [
			'term_taxonomy_id' => $term_tax_ids,
			'hide_empty' => false,
		] );

		$tax_terms_map = [];

		foreach ( $terms as $term ) {
			$taxonomy = $term->taxonomy;
			$tax_terms_map[ $taxonomy ][] = $term->term_id;
		}

		return $tax_terms_map;
	}

	protected function setup_query_authors() {
		if ( $this->is_query_manual() ) {
			return;
		}

		$settings = $this->get_settings_for_display();
		$include_by = $this->get_query_param_by( 'include' );;
		$exclude_by = $this->get_query_param_by( 'exclude' );;
		$include_users = [];
		$exclude_users = [];

		if ( in_array( 'authors', $include_by ) ) {
			$include_users = wp_parse_id_list( $settings['bwdsplx_posts_include_author_ids'] );
		}

		if ( in_array( 'authors', $exclude_by ) ) {
			$exclude_users = wp_parse_id_list( $settings['bwdsplx_posts_exclude_author_ids'] );
			$include_users = array_diff( $include_users, $exclude_users );

		}

		if ( ! empty( $include_users ) ) {
			$this->query_args['author__in'] = $include_users;
		}

		if ( ! empty( $exclude_users ) ) {
			$this->query_args['author__not_in'] = $exclude_users;;
		}
	}


	protected function render () {

		$settings = $this->get_settings_for_display();
		if ( ! $settings['bwdsplx_posts_post_type'] ) {
			return;
		}

		$column = $this->get_column_cls( $settings['column'] );
		$per_page = $this->get_item_per_page('per_page');
		$list_column = $this->get_item_per_page('list_column');
		$bwdsplx_featured_post_column = $this->get_featured_column_cls( $settings['bwdsplx_featured_post_column'] );

		$this->add_render_attribute( 'wrapper', 'class', [ 'bwdsplx-spl-wrapper' ] );

		$this->add_render_attribute(
			'grid_wrap',
			[
				'class' => [
					'bwdsplx-spl-grid-area',
					esc_attr( $column ),
					'yes' === $settings['bwdsplx_make_featured_post'] ? 'bwdsplx-spl-featured-post-on' : 'bwdsplx-spl-featured-post-off'
				],
			]
		);

		$args = $this->get_query_args();

		$args['posts_per_page'] = -1;
		$posts = get_posts( $args );


		if( 'yes' === $settings['bwdsplx_top_bar_show'] ){
			$query_settings = [
				'args' => $args,
				'bwdsplx_posts_post_type' => $settings['bwdsplx_posts_post_type'],
				'per_page' => $per_page,
				'column' => $settings['column'],
				'bwdsplx_make_featured_post' => $settings['bwdsplx_make_featured_post'],
				'bwdsplx_featured_post_column' => $bwdsplx_featured_post_column,
				'bwdsplx_featured_post_style' => $settings['bwdsplx_featured_post_style'],
				'featured_image_size' => $settings['featured_image_size'],
				// 'featured_post_cat' => $settings['featured_post_cat'],
				'bwdsplx_show_badge' => $settings['bwdsplx_show_badge'],
				'bwdsplx_taxonomy_badge' => $settings['bwdsplx_taxonomy_badge'],
				'bwdsplx_featured_post_title' => $settings['bwdsplx_featured_post_title'],
				'bwdsplx_featured_post_title_tag' => $settings['bwdsplx_featured_post_title_tag'],
				'bwdsplx_featured_meta_active' => $settings['bwdsplx_featured_meta_active'],

				'bwdsplx_featured_post_author_icon' => $settings['bwdsplx_featured_post_author_icon'],
				'bwdsplx_featured_post_date_icon' => $settings['bwdsplx_featured_post_date_icon'],
				'bwdsplx_featured_post_comment_icon' => $settings['bwdsplx_featured_post_comment_icon'],
				'bwdsplx_featured_excerpt_length' => $settings['bwdsplx_featured_excerpt_length'],

				'list_column' => $list_column,
				'bwdsplx_list_post_image' => $settings['bwdsplx_list_post_image'],
				'bwdsplx_list_post_image_size' => $settings['bwdsplx_list_post_image_size'],
				'bwdsplx_list_post_title_tag' => $settings['bwdsplx_list_post_title_tag'],
				'bwdsplx_list_meta_active' => $settings['bwdsplx_list_meta_active'],

				'bwdsplx_list_post_author_icon' => $settings['bwdsplx_list_post_author_icon'],
				'bwdsplx_list_post_date_icon' => $settings['bwdsplx_list_post_date_icon'],
				'bwdsplx_list_post_comment_icon' => $settings['bwdsplx_list_post_comment_icon'],
			];
			$query_settings = json_encode( $query_settings, true );

			$this->add_render_attribute( 'wrapper', 'data-settings', $query_settings );
			$this->add_render_attribute( 'wrapper', 'data-total-offset', '0' );
			$this->add_render_attribute( 'wrapper', 'data-offset', $per_page );
		}
		$class_array = [];
		if( 'yes' === $settings['bwdsplx_make_featured_post']) {
			$class_array['featured'] = 'bwdsplx-spl-column bwdsplx-spl-featured-post-wrap '.esc_attr( $bwdsplx_featured_post_column );
			$class_array['featured_inner'] = 'bwdsplx-spl-featured-post '.'bwdsplx-spl-featured-'.esc_attr($settings['bwdsplx_featured_post_style']);
		}

		$loop = 1;
		if ( count( $posts ) !== 0 ) :?>
			<div <?php $this->print_render_attribute_string( 'wrapper' ); ?>>

				<?php $this->render_header_markup( $settings['bwdsplx_top_bar_show'] );?>

				<div <?php $this->print_render_attribute_string( 'grid_wrap' ); ?>>

					<?php self::render_spl_markup( $settings, $posts, $class_array, $list_column, $per_page ); ?>

				</div>
			</div>
		<?php
		else:
			printf( '%1$s %2$s %3$s',
				__( 'No ', 'bwdsplx-post-smart' ),
				esc_html( $settings['bwdsplx_posts_post_type'] ),
				__( 'Found', 'bwdsplx-post-smart' )
			);
		endif;
	}

}